// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file RosenbrockAdaptInstationary.h */

#ifndef AMDIS_ROSENBROCKADAPTINSTATIONARY_H
#define AMDIS_ROSENBROCKADAPTINSTATIONARY_H

#include "AMDiS_fwd.h"
#include "time/RosenbrockStationary.h"
#include "time/RosenbrockMethod.h"
#include "AdaptInstationary.h"
#include "CreatorMap.h"

namespace AMDiS {

  class RosenbrockAdaptInstationary : public AdaptInstationary
  {
  public:
    RosenbrockAdaptInstationary(std::string name, 
				RosenbrockStationary *problemStat,
				AdaptInfo *info,
				ProblemTimeInterface *problemInstat,
				AdaptInfo *initialInfo,
				time_t initialTimestamp = 0);

    /// Runs the Rosenbrock loop until one timestep is accepted.
    void oneTimestep();

  protected:
    /// Pointer to the Rosenbrock method that should be used.
    RosenbrockMethod *rosenbrockMethod;

    /// Pointer to the stationary problem;
    RosenbrockStationary *rosenbrockStat;

    /// Indicates, if this is the very first timestep.
    bool firstTimestep;

    /// If true, the last timestep was rejected.
    bool lastTimestepRejected;


    /// If true, more than one of the last timesteps were rejected.
    bool succRejection;

    /// If greater than 0, than for the first given number of timesteps the timestep
    /// will be not changed and is set to the very first one.
    int fixFirstTimesteps;
    
    /// Timestep of the last accepted time iteration.
    double tauAcc;

    /// Error estimation of the last accepted time iteration.
    double estAcc;

    /// Timestep of the last rejected time iteration.
    double tauRej;

    /// Error estimation of the last rejected time iteration.
    double estRej;

    /// Current timestep.
    double tau;

    /// The value is 1.0 / (tau * gamma), where gamma is a value of the used 
    /// Rosenbrock method.
    double tauGamma;
  };
 
}

#endif // AMDIS_ROSENBROCKADAPTINSTATIONARY_H
